tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/viewerlib/?.lua;"..tmepath.."/contrib/terrameaddons/?.lua"

local rand   = require ("random")
local view   = require ("viewer")
local ding   = require ("dingbats")
local gris   = require ("grissom")
local cf     = require ("cellspace_functions")

-- PARAMETERS
NDIM  = 50   -- cellular space size (NxN)
THRES = 4    -- agent contentedness index: 3 to 5
VAC   = 0.10 -- vacancy ratio: 0.05 to 0.35 (5% to 35%)

STEPS = 3000     -- maximum number of simulation steps

EMPTY  = 0        -- empty cell
ONE    = 1        -- agent prefers one
OTHER  = 2        -- agent prefers other
NVALS  = 3

WAIT     = 50

local wingdings = require ("wingdings")
local util = require ("util")


function update (cs)
    forEachCell (cs, function (cell) 
            if (cell:getAgent() ~= nil ) then
                cell.state = cell:getAgent().preference 
            else
                cell.state = EMPTY
            end
                
    end)
end
-- Auxiliary function 
-- computes whether an agent of a given 'preference' would be satisfied living in a given 'cell'
function  isSatisfied (agent, cell)
	local equal = 0
	forEachNeighbor(cell, function(cell, neigh)
        local other = neigh:getAgent()
		if (other ~= nil) then
			if other.preference == agent.preference then
				equal = equal + 1
			end
		end
	end)
	return equal >= THRES
end

-- 1. Define basic parameters for the model
model = {
    initTime = 1,         -- initial time for the model
    period   = 1,         -- time step for the model
    }
 
 -- 2. Create the cellular space and initialize it
model.cellspace = CellularSpace { xdim = NDIM, ydim = NDIM }

model.cellspace:createNeighborhood { 
            strategy = "moore",
            self     = false,
            wrap     = true 
        }
        
forEachCell (model.cellspace, function (cell) cell.state = EMPTY end)

-- 3. Create the agent
model.agent   = Agent {
                    -- an agent is initialized with a random state (black or red)
                    -- with 50% of probability for each
                    init = function(self)
                        if rand:integer(0, 100) < 50 then
                            self.preference = ONE
                        else
                            self.preference = OTHER
                        end
                    end,
	
                    -- an agent tries to move to a random empty cell whenever it is
                    -- dissatisfied with the current cell it lives
                    execute = function(self)
                        local cell = self:getCell()
                        if not isSatisfied(self, cell) then
                            local newcell = cf.findEmpty (model.cellspace)
                            self:move(newcell)
                        end
                    end
                }
 
-- 4. Create a society of agents
model.society = Society {
                    instance = model.agent,
                    quantity = NDIM * NDIM * ( 1.0 - VAC ),
                    attributes = {"preference"}
                }
   
-- 5. Create the environment
model.env     = Environment { model.cellspace, model.society }
        
model.env:createPlacement {strategy = "random", max = 1}

-- 6. Create a map to view the cellspace
model.map = view.Map {
                subject     =   model.cellspace,
                attributes  =   {"state"},
                colors      =   "Blues",
                values      =   {EMPTY, ONE, OTHER}
            }
-- 7. Create a symbol to display the agents
--[[
 model.sym = view.Symbol { 
                subject     =  model.society,
                attributes  = {"preference"},
                observer    =   model.map,
                colors      = "GnBu",
                font        = "Wingdings",
                fontSize    = 22,
                symbol      = wingdings.SMILEFACE,
                values      = {EMPTY, ONE, OTHER}
            }
--]]
-- 8. Create a timer to control the model
model.timer = Timer {
    		Event {time = 1, period = 1, 
                    action = function (ev)
                                model.society:execute()
                                update (model.cellspace)
                                model.cellspace:notify ()
                                util.wait(WAIT)
                    end
                    }
            }
    
model.timer:execute (STEPS) 


