
--- Visualisation library for TerraME
--  This module implements three different types of viewers for TerraME
--  A Plot is a 1D chart, useful to display time series 
--  A Map  is a 2D chart, useful to display cell space
--  A Symbol is a set of symbols, useful to display agents
--  @author Gilberto Camara, 2014

--- Configuration variable for contributed TerraME libraries
--  @usage tmepath should point to the TerraME installation directory
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/viewerlib/?.lua;"

--- Dependencies of the viewer
--  @usage ding calls the dingbats module
local ding = require ("dingbats")
--  @usage colorbar calls the "colorbar"  module
local colorbar = require ("colorbar")

local view = {}

--- support for a plot viewer
__Plot = {
        xLabel      = "time",
        type        = "chart",
        
        mkPlotObserver = function (self)
            local obs = Observer {
                    subject     = self.subject,
                    attributes  = self.attributes,
                    curveLabels = self.curveLabels,
                    title       = self.title,
                    xLabel      = self.xLabel,
                    yLabel      = self.yLabel,
                    type        = self.type
                }
            return obs
        end
    }

-- defines the metatable for the plot
metaTablePlot = {__index = __Plot}

--- Creates a plot viewer
--  Takes a table as input parameter
--  @param argv - a table with the description of the plot
--  @param argv.subject      is the TerraME object that will be plotted (mandatory)
--  @param argv.attributes   are the names of the attributes of the subject that will be shown (mandatory)
--  @param argv.curveLabels  are the labels of the plot (optional)
--  @param argv.title        is the title of the plot (optional)
--  @param agrv.xLabel       is the label of the X axis (optional, default is "time")
--  @param argv.yLabel       is the label of the Y axis (optional)
function view.Plot(argv)
	-- the model first gets the scenario-specific parameters
	local plot = argv
    
    -- at least subject and attributes must be provided
    assert (plot.subject    ~= nil, "Viewer: a plot must have a subject" )
    assert (plot.attributes ~= nil, "Viewer: a plot must have attributes")
    
	-- all other parameters are accessed via the metatable
	setmetatable(plot, metaTablePlot)
    
    -- the default is use the attribute name for the title and the labels
    if (plot.curveLabels == nil) then plot.curveLabels = plot.attributes   end
    
    -- create the observer
    local obs = plot:mkPlotObserver ()
return plot
end

--- support for a symbol viewer
__Symbol = {
        grouping    = "uniquevalue",
        colors      = "Blues",
        font        = "Zapf Dingbats",
        fontSize    = 24,
        symbol      = ding.OPEN_CENTRE_TEARDROP_SPOKED_ASTERISK,
        type        = "map",
        
        symbolObserver = function (self)
                 assert (self.grouping ~= nil, "Viewer: grouping must be provided")
                 assert (self.colorBar ~= nil, "Viewer: colorBar is missing")
                 local legend = Legend {
                         grouping = self.grouping,
                         colorBar = self.colorBar,
                         font      = self.font,
                         fontSize  = self.fontSize,
                         symbol    = self.symbol
                     }
                 if (self.grouping ~= "uniquevalue") then
                     assert (self.minimum ~= nil, "Viewer: minimum attribute value is missing")
                     assert (self.maximum ~= nil, "Viewer: maximum attribute value is missing")
                     assert (self.slices  ~= nil, "Viewer: number of slices is missing")
                     legend.minimum = self.minimum
                     legend.maximum = self.maximum
                     legend.slices  = self.slices
                end
                local obs = Observer {
                    subject     =   self.subject,
                    attributes  =   self.attributes,
                    observer    =   self.observer,
                    legends     =   {legend},
                    type        =   "map"
                }
                return obs
        end
    }

-- defines that Sugarscape (see above) is a metatable 
metaTableSymbol = {__index = __Symbol}

--- Creates a symbol viewer
--  Takes a table as input parameter
--  @param argv - a table with the description of the plot
--  @param argv.subject      is the TerraME object that will be plotted (mandatory)
--  @param argv.attributes   are the names of the attributes of the subject that will be shown (mandatory)
--  @param argv.observer     is the observer where the symbols will be shown (mandatory)
--  @param argv.values       the set of values that will receive colors (mandatory)
--  @param argv.colors       a valid set of color names for the Color Brewer (optional, default is "Blues")
--  @param argv.labels       the labels of the values (optional)
--  @param argv.font         the font that provides the symbol (optional, default is "Zapf Dingbats")
--  @param argv.fontSize     is the size of the symbol  (optional, default is 24)
--  @param argv.symbol       is the ASCII code for the symbol (optional, default is ASTERISK)
function view.Symbol(argv)
	-- the model first gets the scenario-specific parameters
	local sym = argv
    -- at least subject, attribute, observer and values must be provided
    assert (sym.subject    ~= nil, "Viewer: subject of symbol must be provided")
    assert (sym.attributes ~= nil, "Viewer: attributes of symbol must be provided") 
    assert (sym.observer   ~= nil, "Viewer: observer of symbol must be provided")
    -- other parameters are accessed via the metatable
	setmetatable(sym, metaTableSymbol)
    -- create a colorBar for the symbol
    sym.colorBar = colorbar.make (sym)
    -- create the observer
    local obs    = sym:symbolObserver ()
return obs
end

--- support for a map viewer
__Map = {
        grouping    = "uniquevalue",
        colors      = "Greys",
        type        = "map",
        
        mapObserver = function (self)
                        local legend = {}
                        assert (self.grouping ~= nil, "Viewer: grouping must be provided")
                        assert (self.colorBar ~= nil, "Viewer: colorBar is missing")
                        if (self.grouping == "uniquevalue") then
                            legend = Legend {
                                        grouping = self.grouping,
                                        colorBar = self.colorBar
                            }
                        else 
                            assert (self.minimum ~= nil, "Viewer: minimum attribute value is missing")
                            assert (self.maximum ~= nil, "Viewer: maximum attribute value is missing")
                            assert (self.slices  ~= nil, "Viewer: number of slices is missing")
                            legend  = Legend {
                                        grouping  = self.grouping,
                                        minimum   = self.minimum,
                                        maximum   = self.maximum,
                                        slices    = self.slices,
                                        colorBar  = self.colorBar
                            }
                        end
                        local obs = Observer {
                            subject     =  self.subject,
                            attributes  =  self.attributes,
                            legends     =  {legend},
                            type        =  self.type
                        }
                        return obs
        end
}

-- defines that __Map is a metatable 
metaTableMap = {__index = __Map}

--- Creates a map viewer
--  Takes a table as input parameter
--  @param argv - a table with the description of the map
--  @param argv.subject      is the TerraME object that will be plotted (mandatory)
--  @param argv.attributes   are the names of the attributes of the subject that will be shown (mandatory)
--  @param argv.values       the set of values that will receive colors (mandatory for unique values)
--  @param argv.colors       a valid set of color names for the Color Brewer (optional, default is "Blues")
--  @param argv.labels       the labels of the values (optional)
--  @param argv.slices       number of slices (mandatory for "equalsteps" or "quantil")
--  @param argv.minimum      the minimum value to be plotted (mandatory for equal steps or quantiles)
--  @param argv.maximum      the maximum value to be plotted (mandatory for equal steps or quantiles)
function view.Map(argv)
	-- first gets the user-defined parameters
	local map = argv
    -- at least subject and attribute must be provided
    assert (map.subject    ~= nil, "Viewer: subject of map must be provided")
    assert (map.attributes ~= nil, "Viewer: attributes of map subject must be provided") 
    
	-- all other parameters are accessed via the metatable
	setmetatable(map, metaTableMap)
    
    -- create the colorBar
    map.colorBar = colorbar.make (map)

    -- create the observer for the map
    local obs    = map:mapObserver ()

return obs
end

return view
